import java.util.HashMap;

    // Maintain a collection of SimpleShips, accessed by their name attribute.
    // This version enforces a maximum size on the collection.
class ShipCollection {
        // Add a ship to the collection. The ship's
        // name is used as a key.
        // Enforce a (variable) capacity on the collection - maxSize.
    public void addShip(SimpleShip ship){
        String name = ship.getName();
        if((name != null) && (name.length() > 0)){
            HashMap ships = getShips();
            if(ships.get(name) == null){
                // No ship with matching name already in the collection.
                // Check that there is space available.
                if(getSize() < getMaxSize()){
                    ships.put(name,ship);
                }
                else{
                    System.err.println("The collection is full: "+
                                       getMaxSize()+" ships.");
                    System.err.println("Unable to add: "+name);
                }
            }
            else{
                System.err.println("A ship named "+name+
                                   " already exists.");
            }
        }
        else{
            System.err.println("Unable to add a ship with no name.");
        }
    }

        // Return a reference to the ship with the given name.
        // Return null if none is found.
    public SimpleShip getShip(String name){
        SimpleShip s = (SimpleShip) getShips().get(name);
        if(s == null){
            System.err.println("No ship called "+name+" was found.");
        }
        return s;
    }

        // Remove the ship with the given name. Return a
        // reference to it. Return null if no ship with the
        // given name is found.
    public SimpleShip removeShip(String name){
        SimpleShip s = (SimpleShip) getShips().remove(name);
        if(s == null){
            System.err.println("No ship called "+name+" was found.");
        }
        return s;
    }

        // Return the existing number of ships.
    public int getSize(){
        return getShips().size();
    }

    public int getMaxSize(){
        return maxSize;
    }

        // Set a maximum size for the collection. This
        // should be consistent with the existing contents.
    public void setMaxSize(int max){
        if(max >= 0){
            // Prevent a reduction below the existing size.
            int size = getSize();
            if(max >= size){
                maxSize = max;
            }
            else{
                System.err.println(
                        "Max cannot be reduce below existing size.");
            }
        }
        else{
            System.out.println("Illegal maximum: "+max);
        }
    }

    private HashMap getShips(){
        return ships;
    }

    // Limit the number of ships that can be stored in this collection.
    private int maxSize = 20;
    private final HashMap ships = new HashMap();
}
