        // Provide for storage and retrieval of a limited
        // number of Car objects.
class CarPark {
        // Create parking spaces for capacity cars.
    public CarPark(int capacity){
        // Create locations for the Cars to be stored.
        places = new Car[capacity];
        // Initialize the number of free spaces.
        setSpaces(capacity);
    }

        // How many free spaces?
    public int getSpaces(){
        return spaces;
    }

        // Is the car park full?
    public boolean full(){
        return getSpaces() == 0;
    }

        // Park the given car, and return its location number.
        // Throw an exception if there is no free location.
    public int park(Car c) throws Exception {
        if(!full()){
            // Look for the lowest number location.
            Car[] places = getPlaces();
            final int numPlaces = places.length;
            for(int location = 0; location < numPlaces; location++){
                if(places[location] == null){
                    // Found a space.
                    places[location] = c;
                    setSpaces(getSpaces()-1);
                    return location;
                }
            }
            // This point should never be reached.
            throw new Exception("Internal logic failure in park.");
        }
        else{
            // No free location.
            throw new Exception(
                        "Parking failure: the car park is full.");
        }
    }

        // Collect the car from the given location.
        // Throw an exception if the location is invalid,
        // or unoccupied.
    public Car collect(int location) throws Exception {
        Car[] places = getPlaces();
        if((location >= 0) && (location < places.length)){
            if(places[location] != null){
                Car c = places[location];
                // Free the location.
                places[location] = null;
                setSpaces(getSpaces()+1);
                return c;
            }
            else{
                throw new Exception("Unoccupied location: "+location);
            }
        }
        else{
            throw new Exception("Out of bounds index: "+location);
        }
    }

    private Car[] getPlaces(){
        return places;
    }

    private void setSpaces(int s){
        spaces = s;
    }

    private final Car[] places;
    // How many free spaces.
    private int spaces = 0;
}
