import java.io.*;
import java.util.ArrayList;

    /* Maintain a File object relating to a particular directory (folder).
     * Construction will fail if either the given
     * directory name does not exist or the name does not relate to a
     * directory. Methods are provided to print a simple list of the
     * directory's contents, or a more detailed list including the type
     * and permissions for each file, the date on which it was last
     * modified and its name.
     * printNestedContents() prints the contents of this directory and
     * those of all subdirectories.
     */

class RecursiveLister extends DirectoryLister {
        // Construct from the name of a directory.
    public RecursiveLister(String dirname)
                        throws FileNotFoundException, Exception {
        super(dirname);
    }

        // Construct from a File object that should refer to a directory.
    public RecursiveLister(File dir)
                        throws FileNotFoundException, Exception {
        super(dir);
    }

        /* Print the contents of this directory and all nested
         * directories. This is not handled by recursion of this method
         * but by recursively creating further RecursiveLister
         * objects for each sub directory.
         */
    public void printNestedContents(){
        // Print the contents of this directory.
        printContents();
        // Find each directory in the current directory.
        File[] directories = getSubDirectories();
        final int numDirectories = directories.length;
        for(int i = 0; i < numDirectories; i++){
            try{
                RecursiveLister subdir = new RecursiveLister(directories[i]);
                subdir.printNestedContents();
            }
            catch(Exception e){
                System.err.println(e);
            }
        }
    }

        // Return an array of File objects representing the
        // sub-directories within this directory.
        // Return an array of all the subdirectories within
        // this directory.
    public File[] getSubDirectories(){
        ArrayList directories = new ArrayList();
        File details = getDirectory();
        final String path = getPath();
        String[] list = details.list();
        final int numFiles = list.length;
        for(int i = 0; i < numFiles; i++){
            File f = new File(path+File.separator+list[i]);
            if(f.isDirectory()){
                directories.add(f);
            }
        }
        // Return the directories collection as an array.
        File[] fileArray = new File[directories.size()];
        directories.toArray(fileArray);
        return fileArray;
    }
}
