import java.util.*;

        // Provide forward and backward iteration over an
        // array of objects. Only a partial implementation of
        // ListIterator is provided.
class ArrayIterator2 implements Iterator, ListIterator {
        // Retain a reference to the data to be iterated over.
    public ArrayIterator2(Object[] data){
        array = data;
    }

    public boolean hasNext(){
        return getLastIndex()+1 < getArray().length;
    }

    public Object next() throws NoSuchElementException {
        if(hasNext()){
            int index = getLastIndex()+1;
            // Update the value for next time.
            setLastIndex(index);
            return getArray()[index];
        }
        else{
            throw new NoSuchElementException();
        }
    }

    public boolean hasPrevious(){
        return getLastIndex() >= 0;
    }

    public Object previous() throws NoSuchElementException {
        if(hasPrevious()){
            int index = getLastIndex();
            setLastIndex(index-1);
            return getArray()[index];
        }
        else{
            throw new NoSuchElementException();
        }
    }

        // Remove an element by setting it to null.
    public void remove() throws IllegalStateException,
                                UnsupportedOperationException {
        int index = getLastIndex();
        if(index < 0){
            throw new IllegalStateException();
        }
        else{
            getArray()[index] = null;
        }
    }

        // Unimplemented methods.
    public void add(Object o){
        throw new UnsupportedOperationException("add");
    }

    public void set(Object o){
        throw new UnsupportedOperationException("set");
    }

    public int previousIndex(){
        throw new UnsupportedOperationException("previousIndex");
    }

    public int nextIndex(){
        throw new UnsupportedOperationException("nextIndex");
    }

        // Accessors and mutators for the lastIndex and array attributes.
    private int getLastIndex(){
        return lastIndex;
    }

    private void setLastIndex(int n){
        lastIndex = n;
    }

    private Object[] getArray(){
        return array;
    }

    // The array of objects to be iterated over.
    private final Object[] array;
    // The index of the last element returned.
    private int lastIndex = -1;
}
