        // Implement a strategy that always plays high if a card
        // of the correct suit is available. Otherwise, play any card.
class PlayHigh implements CardPlayer {
    public PlayHigh(int id){
        // The player has an identity.
        identity = id;
    }

        // Receive a card from the dealer.
        // Keep the hand sorted.
    public void takeCard(SuitedCard c){
        Hand hand = getHand();
        hand.add(c);
        hand.sort();
    }

        // We choose to ignore this, which means we will have
        // an incomplete picture of which cards remain in the
        // other players' hands.
    public void undealtCard(SuitedCard c){
    }

        // Play any card from the hand. Return null if
        // we have no more cards. We have a free choice
        // of suit.
    public SuitedCard playAnyCard() throws RuntimeException {
        // Choose the highest card from an available suit.
        final String[] suits = SuitedCard.getSuitNames();
        for(int i = 0; i < suits.length; i++){
            SuitedCard card = chooseCardInSuit(suits[i]);
            if(card != null){
                return card;
            }
        }
        throw new RuntimeException("No cards available.");
    }

        // We must follow suit, if possible.
    public SuitedCard followSuit(String suit){
        SuitedCard card = chooseCardInSuit(suit);
        if(card != null){
           return card;
        }
        else{
            return playAnyCard();
        }
    }

        // Receive notification of a card played in this round.
        // This could be the card that we have just played.
    public void viewCard(SuitedCard c){
    }

        // Notification of who won the round just completed.
        // This could be us.
    public void roundWinner(int id){
    }

    public int getIdentity(){
        return identity;
    }

    public int numCards(){
        return getHand().numCards();
    }

        // Select a card in the given suit, if any.
        // Return null if none is available.
    private SuitedCard chooseCardInSuit(String suit){
        final SuitedCard[] cards = getHand().getCardsInSuit(suit);
        if(cards.length != 0){
            // Play the first, which will be the highest.
            getHand().remove(cards[0]);
            return cards[0];
        }
        else{
            return null;
        }
    }

    private Hand getHand(){
        return hand;
    }

    // A collection to hold the player's cards.
    private Hand hand = new Hand();
    // The player's identifying number.
    private final int identity;
}
